#!/system/bin/sh
#
#H# enable_wireless_adb.sh - enable wireless adb on a phone running the Android OS 
#H#
#H# Usage:
#H#
#H#    enable_wireless_adb.sh
#H#
#H#    The script will wait up to 60 seconds until the settings command is usable and then enable
#H#    the development option and the adb wifi connection.
#H#
#H#    The script will do nothing if the file "/cache/enable_wireless_adb_disabled" exists.
#H#    The script will write the messages into the file "/cache/enable_wireless_adb.log" if running in a session without tty
#H#
#H#    The script must run on the phone while the Android OS is running
#H#
#H#    The script can be used as a script in service.d for Magisk.
#H#
#H#    The script can run either as root or no root user; if running as non-root user without su support the script
#H#    can not correct the adbd keystore or detect the port used by the adbd
#H#
#H#    The script assumes that adb via USB is already activated and the adbd keystore is already configured with the rsa key for the host
#H#
#
# History
#   15.01.2024 1.0.0.0 /bs
#     initial release

# for debugging:
#
# set -x

# define some constants
#
__TRUE=0
__FALSE=1

# semaphor file - the script will do nothing if this file exists
#
SEMFILE="/cache/enable_wireless_adb_disabled"

#
# logfile to use if running in a session without tty
#
LOGFILE="/cache/enable_wireless_adb.log"

# return code 
#
THISRC=${__TRUE}

# help variable for the script flow
#
CONT=${__TRUE}

# maximum time in seconds to wait until the command "settings" work
#
MAX_WAIT_TIME_IN_SECONDS=60

# the user running this script
#
CUR_USER="$( id -un )"

ROOT_ACCESS_OKAY=${__TRUE}

SU_PREFIX=""

# the IP in the WLAN on the running Android OS 
#
CUR_IP="$( ip addr list | grep -e "global wlan0" | awk '{ print $2}' | cut -f1 -d "/" )"

# CUR_IP="$( ifconfig wlan0 | grep "inet addr" | awk '{ print $2}' | cut -f2 -d ":" )"

# the BSSID of the currently connected WLAN
#
WIFI_BSSID="$( dumpsys wifi | grep mLastBssid | awk '{ print $2}'   )"

# for testing only
#
if [ "${WIFI_BSSID}"x = ""x ] ; then
	echo "WLAN not up yet - using hardcoded BSSID"
	WIFI_BSSID="24:4b:fe:f3:02:58"
fi

#
# executable to change the settings
#
SETTINGS="/system/bin/settings"

# this variable will be set to true if the settings binary is working
#
SETTINGS_IS_WORKING=${__FALSE}


# directory with the adbd keystore
#
ADBD_KEYSTORE_DIRECTORY="/data/misc/adb"

# temporary files used to check and correct the adbd keystore
#
ADB_TEMP_KEYSTORE="${ADBD_KEYSTORE_DIRECTORY}/adb_temp_keys.xml"
ADB_TEMP_KEYSTORE_PLAIN_XML="${ADB_TEMP_KEYSTORE}.plain.$$"
ADB_TEMP_KEYSTORE_NEW="${ADB_TEMP_KEYSTORE}.new.$$"
ADB_TEMP_KEYSTORE_BACKUP="${ADB_TEMP_KEYSTORE}.backup.$$"

ADB_WIFI_IS_ALREADY_ENABLED=${__FALSE}

ENABLING_ADB_WIFI_WAS_SUCCESSFULL=${__FALSE}

TMP_SCRIPT="/cache/conv_xml.$$.sh"

# ---------------------------------------------------------------------
#
echo "$0 -- enabling adb connections via WLAN"


#
# check if we're running in a session with tty
#
/system/bin/tty -s 
if [ $? -eq 0 ] ; then 
  RUNNING_WITH_TTY_SESSION=${__TRUE}
  echo "Running in a session with tty "
else
  echo "Running in a session without tty -- using the logfile \"${LOGFILE}\" "
  exec 2>"${LOGFILE}" 1>&2
  RUNNING_WITH_TTY_SESSION=${__FALSE}
fi

if [ "${CUR_USER}" != "root"x ] ; then
  echo "Running as non-root user"
  CUR_OUTPUT="$( su - -c "id -un" 2>/dev/null )"
  if [ "${CUR_OUTPUT}"x != "root"x ] ; then
    echo "WARNING: root access not supported -- can not correct the adbd keystore and can not check the port used by the adbd"
    ROOT_ACCESS_OKAY=${__FALSE}
  else
    SU_PREFIX="su - -c "  
  fi
fi


#
# check if the sempahor file exists
#
if [ -r "${SEMFILE}" ] ; then
  if [ ${RUNNING_WITH_TTY_SESSION} = ${__TRUE} ] ; then
    echo "WARNING: The file \"${SEMFILE}\" exists but we're running in an tty session"
  else
    echo "$0: The file \"${SEMFILE}\" exists -- will do nothing "
    THISRC=${__TRUE}
    CONT=${__FALSE}
  fi
fi

if [ ${THISRC} = ${__TRUE} -a ${CONT} = ${__TRUE} ] ; then
  echo "Waiting up to ${MAX_WAIT_TIME_IN_SECONDS} seconds until the settings command work ..."

  i=0
  while [ $i -lt ${MAX_WAIT_TIME_IN_SECONDS} ] ; do
    if [ -x "${SETTINGS}" ] ; then
      "${SETTINGS}" get global development_settings_enabled 2>/dev/null  1>/dev/null 
      if [ $? -eq 0 ] ; then
        SETTINGS_IS_WORKING=${__TRUE}
        break
      fi
    fi
    echo "Waiting 5 seconds ..."
    let i=i+5
    sleep 5
  done

  if [ ${SETTINGS_IS_WORKING} != ${__TRUE} ] ; then
    echo "ERROR: The command settings still does not work - probably you should increase the  wait time; current value is ${MAX_WAIT_TIME_IN_SECONDS} seconds"
    CONT=${__FALSE}
    THISRC=${__FALSE}
  else
    echo "The time to wait until the command \"${SETTINGS}\" works was ${i} seconds"
  fi  
fi

if [ ${THISRC} = ${__TRUE} -a ${CONT} = ${__TRUE} ] ; then

  echo "Retrieving the current values for the adb connections ..."
  
  ADB_ENABLED="$( "${SETTINGS}" get global adb_enabled )"

  ADB_WIFI_ENABLED="$( "${SETTINGS}" get global adb_wifi_enabled )"

  echo "The current value for the setting \"global adb_enabled\" is : ${ADB_ENABLED}"
  echo "The current value for the setting \"global adb_wifi_enabled\" is : ${ADB_WIFI_ENABLED}"

  if [ "${ADB_WIFI_ENABLED}"x = "1"x ] ; then
    echo "The adb WIFI connection is already enabled"
    ADB_WIFI_IS_ALREADY_ENABLED=${__TRUE}

    THISRC=${__TRUE}
    CONT=${__FALSE}
  fi
fi

if [ ${THISRC} = ${__TRUE} -a ${CONT} = ${__TRUE} ] ; then

  DEVELOPMENT_SETTINGS_ENABLED="$( "${SETTINGS}" get global development_settings_enabled )"
  
  echo "The current value for the setting \"global development_settings_enabled\" is : ${DEVELOPMENT_SETTINGS_ENABLED}"

  if [ "${DEVELOPMENT_SETTINGS_ENABLED}"x = "1"x ] ; then
    echo "The development settings are already enabled"
  else
    echo "Now enabling the development settings ...."
    "${SETTINGS}" put global development_settings_enabled 1 
    if [ $? -ne 0 ] ; then
      echo "ERROR: Something went wrong enabling the development settings"
      THISRC=${__FALSE}
      CONT=${__FALSE}
    fi
  fi
fi



if [ ${THISRC} = ${__TRUE} -a ${CONT} = ${__TRUE} ] ; then

  echo "Checking if the current WLAN is already configured in the ADBD keystore ..."

  if [ "${WIFI_BSSID}"x = ""x ] ; then
    echo "Can not detect the BSSID of the connected WLAN"
  else
    echo "The BSSID of the connected WLAN is \"${WIFI_BSSID}\" "
  fi

  if [ "${WIFI_BSSID}"x = ""x ] ; then
    :
  elif [ ${ROOT_ACCESS_OKAY} != ${__TRUE} ]; then
    echo "WARNING: Can neither read or write the adbd keystore without root access"
  elif ! ${SU_PREFIX} test -r "${ADB_TEMP_KEYSTORE}" ; then
    echo "WARNING: The adbd keystore \"${ADB_TEMP_KEYSTORE}\" does not yet exist"
  else
    echo "Converting the file \"${ADB_TEMP_KEYSTORE}\" to a plain xml file \"${ADB_TEMP_KEYSTORE_PLAIN_XML}\" ..."
    ${SU_PREFIX} abx2xml "${ADB_TEMP_KEYSTORE}" "${ADB_TEMP_KEYSTORE_PLAIN_XML}"
    if [ $? -ne 0 ] ; then
      echo "ERROR: Error converting the file \"${ADB_TEMP_KEYSTORE}\" to a plain xml file \"${ADB_TEMP_KEYSTORE_PLAIN_XML}\""
      ${SU_PREFIX} \rm -f "${ADB_TEMP_KEYSTORE_PLAIN_XML}" 2>/dev/null
    else
      echo "Checking if the BSSID \"${WIFI_BSSID}\" is already configured in the adbd temporary keystore ..."
      
      ADBD_KEYSTORE_ALREADY_OKAY=${__TRUE}

      NEW_ADBD_KEYSTORE_ENTRIES=""

if [ 1 = 0 ] ; then
      ${SU_PREFIX} grep "adbKey key" "${ADB_TEMP_KEYSTORE_PLAIN_XML}" | grep "${CURRENT_PUB_KEY}" >/dev/null
      if [ $? -eq 0 ] ; then
        echo "The current public key is already configured in the adb temporary keystore "
        NEW_ADBD_KEYSTORE_ENTRIES="${NEW_ADBD_KEYSTORE_ENTRIES}
<adbKey key=\"${CURRENT_PUB_KEY}\""
      else
        ADBD_KEYSTORE_ALREADY_OKAY=${__FALSE}
      fi
fi
            
      ${SU_PREFIX} grep "${WIFI_BSSID}" "${ADB_TEMP_KEYSTORE_PLAIN_XML}"
      if [ $? -eq 0 ] ; then
        echo "The BSSID \"${WIFI_BSSID}\" is already configured in the adb temporary keystore "
      else
        ADBD_KEYSTORE_ALREADY_OKAY=${__FALSE}
      fi

     NEW_ADBD_KEYSTORE_ENTRIES="$( echo "${NEW_ADBD_KEYSTORE_ENTRIES}" | egrep -v "^$" )"

       echo "New entries for the keystore: "
       echo "---"
       echo "${NEW_ADBD_KEYSTORE_ENTRIES}"
       echo "---"
      chmod 0600 /data/misc/adb/adb_temp_keys.xml
      chown system:shell /data/misc/adb/adb_temp_keys.xml
      chcon -v u:object_r:adb_keys_file:s0 /data/misc/adb/adb_temp_keys.xml   
      if [ ${ADBD_KEYSTORE_ALREADY_OKAY} = ${__TRUE} ] ; then
        ${SU_PREFIX} \rm -f "${ADB_TEMP_KEYSTORE_PLAIN_XML}" 2>/dev/null
      else
        echo "The BSSID \"${WIFI_BSSID}\" is not yet configured in the adb temporary keystore - will add it now ...."
          cat <<EOT >"${TMP_SCRIPT}"
          set -x
          ps -ef | grep adbd
          stop adbd
          ps -ef | grep adbd
          sed -i "s#</keyStore>#  <wifiAP bssid=\"${WIFI_BSSID}\" />\n</keyStore>#g" "${ADB_TEMP_KEYSTORE_PLAIN_XML}" 
          unix2dos "${ADB_TEMP_KEYSTORE_PLAIN_XML}" 
          xml2abx "${ADB_TEMP_KEYSTORE_PLAIN_XML}" "${ADB_TEMP_KEYSTORE_NEW}" 
          cp "${ADB_TEMP_KEYSTORE}" "${ADB_TEMP_KEYSTORE_BACKUP}"
          cp "${ADB_TEMP_KEYSTORE_NEW}" "${ADB_TEMP_KEYSTORE}"  
          chmod 0600 "${ADB_TEMP_KEYSTORE}"
          chown system:shell "${ADB_TEMP_KEYSTORE}"
          chcon -v u:object_r:adb_keys_file:s0 "${ADB_TEMP_KEYSTORE}"
          ps -ef | grep adbd
          start adbd
          ps -ef | grep adbd
EOT
        ${SU_PREFIX} sh "${TMP_SCRIPT}"

        if [ $? -ne 0 ] ; then
          echo "WARNING: Error adding the BSSID \"${WIFI_BSSID}\" to the xml file"
#          CONT=${__FALSE}
        fi
      fi
    fi
  fi
fi



if [ ${THISRC} = ${__TRUE} -a ${CONT} = ${__TRUE} ] ; then

  echo "Now enabling the adb WIFI connection  ...."
  "${SETTINGS}" put global adb_wifi_enabled 1 
  if [ $? -ne 0 ] ; then
    echo "${CUR_OUTPUT}"
    echo "ERROR: Something went wrong enabling the adb WIFI connection"
    THISRC=${__FALSE}
  else
    echo "Enabling the adb WIFI connection was successfull"

    echo "Re-reading the values for the changed settings now ..."
  
    NEW_ADB_WIFI_ENABLED="$( "${SETTINGS}" get global adb_wifi_enabled )"
    echo "The current value for the setting \"global adb_wifi_enabled\" is now : ${NEW_ADB_WIFI_ENABLED}"
        
    ENABLING_ADB_WIFI_WAS_SUCCESSFULL=${__TRUE}
  fi

fi

if [ ${ENABLING_ADB_WIFI_WAS_SUCCESSFULL} = ${__TRUE} -o ${ADB_WIFI_IS_ALREADY_ENABLED} = ${__TRUE} ] ; then
  echo "Retrieving the port used for the adbd wifi connection ..."
  CUR_OUTPUT="$(  ${SU_PREFIX} netstat -alnp 2>/dev/null | grep adb | grep " LISTEN "  )"
   if [ "${CUR_OUTPUT}"x = ""x ] ; then
     echo "WARNING: Can not detect the port used by the adbd -- please check the GUI on the phone for a confirmation request"
   else
     CUR_ADBD_PORT="$( echo "${CUR_OUTPUT}" | awk '{ print $4}' | awk -F:  '{ print $NF}' )"
     if [ "${CUR_ADBD_PORT}"x = ""x ] ; then
       echo "WARNING: Can not detect the port used by the adbd -- please check the GUI on the phone for a confirmation request"
     else
       echo "The adbd is listening on the port ${CUR_ADBD_PORT}"
       echo "Use

adb -e -L tcp:localhost:5237 connect ${CUR_IP}:${CUR_ADBD_PORT}

to connect to the adbd via WIFI (5237 can be any free port on the PC)"
     fi
   fi
fi
 
if [ ${RUNNING_WITH_TTY_SESSION} = ${__FALSE} ] ; then

  if [ ${ENABLING_ADB_WIFI_WAS_SUCCESSFULL} = ${__TRUE} ] ; then
    echo "Enabling adb was successfull - will now disable the automatic start of this script"
#
# disable the automatic start of the script for the next reboots if executed via Magisk
#
    touch "${SEMFILE}"
  else
    echo "Enabling adb WIFI was not successfull - will not disable the automatic start of this script"
  fi
fi

exit 0
